import serial, time, datetime, signal, threading, string, random, sys
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.animation as animation

def exit_gracefully(signum, frame):
    signal.signal(signal.SIGINT, original_sigint)
    global run
    run.clear()

    signal.signal(signal.SIGINT, exit_gracefully)

fig, ax = plt.subplots(figsize=(18, 5))

if len(sys.argv) < 2:
    print('usage: {} COMx'.format(sys.argv[0]))
    exit(-1)

port = sys.argv[1]

vis_length = 60 * 40
#vis_screenshot_trigger_us = 250000
vis_screenshot_trigger_bytes = 1024
vis_points = np.zeros(int(vis_length)).astype(np.int)
vis_time = -np.arange(0, len(vis_points))[::-1]
save_fig_time = None

def update(i):
    global vis_points
    global save_fig_time
    global port

    if save_fig_time is not None:
        if datetime.datetime.now() >= save_fig_time:
            save_fig_time = None
            plt.savefig(datetime.datetime.now().strftime('%Y-%m-%d_%H_%M_%S_{}_plot.png'.format(port)))
            print("{}: stored figure".format(datetime.datetime.now().strftime('%Y-%m-%d_%H:%M:%S')))

    ax.clear()
    ax.plot(vis_time, vis_points)
    ax.set(xlabel='Time [s]', ylabel='Bytes', title='{} receive lengths'.format(port))
            
    #print(vis_points)

def loopback_func(run, interval, s_in, csv_out):
    global vis_points
    global save_fig_time

    while run.is_set():
        #ts = datetime.datetime.now()
        r = s_in.read(1024*1024)
        if r is None:
            length = 0
        else:
            length = len(r)
        #print("{}: {}".format(datetime.datetime.now().strftime('%Y-%m-%d_%H_%M_%S_plot.png'), length))
        #t_delta = datetime.datetime.now() - ts
        if ((length >= vis_screenshot_trigger_bytes) or length == 0) and save_fig_time is None:
            save_fig_time = datetime.datetime.now() + datetime.timedelta(seconds=40)

        csv_out.write('{}, {}\n'.format(datetime.datetime.now().strftime('%H:%M:%S.%f'), length))
        vis_points = np.roll(vis_points, -1)
        vis_points[-1] = length

s_in = serial.Serial(port, 115200, timeout=1.0)

csv_out = open(datetime.datetime.now().strftime('%Y-%m-%d_%H_%M_%S_{}_receive_lengths.csv'.format(port)), 'w')
csv_out.write('Timestamp, bytes read,\n')

time.sleep(1.0)

original_sigint = signal.getsignal(signal.SIGINT)
signal.signal(signal.SIGINT, exit_gracefully)

run = threading.Event()
run.set()

threading.Thread(target=loopback_func, args=(run, 1, s_in, csv_out)).start()

anim = animation.FuncAnimation(fig, update, interval=100)
plt.tight_layout()
plt.show()
run.clear()
#print('Running loopback. Press Ctrl+c to stop.')


#print('')

#print(time_deltas)